@@ -38,7 +38,6 @@ exception statement from your version. */
 
 package gnu.CORBA.CDR;
 
-import java.io.DataInput;
 import java.io.EOFException;
 import java.io.FilterInputStream;
 import java.io.IOException;
@@ -55,7 +54,7 @@ import java.io.PushbackInputStream;
  */
 public class LittleEndianInputStream
   extends FilterInputStream
-  implements abstractDataInputStream
+  implements AbstractDataInput
 {
   // Byte buffer, used to make primitive read calls more efficient.
   byte[] buf = new byte[ 8 ];
