@@ -38,6 +38,9 @@ exception statement from your version. */
 
 package gnu.java.awt.peer.gtk;
 
+import gnu.classpath.Configuration;
+import gnu.classpath.Pointer;
+
 import gnu.java.awt.ClasspathToolkit;
 import gnu.java.awt.peer.ClasspathFontPeer;
 import gnu.java.awt.font.opentype.NameDecoder;
@@ -164,7 +167,10 @@ public class GdkFontPeer extends ClasspathFontPeer
 
   static 
   {
-    System.loadLibrary("gtkpeer");
+    if (Configuration.INIT_LOAD_LIBRARY)
+      {
+        System.loadLibrary("gtkpeer");
+      }
 
     initStaticState ();
 
@@ -172,6 +178,14 @@ public class GdkFontPeer extends ClasspathFontPeer
 
   private ByteBuffer nameTable = null;
 
+  /**
+   * The pointer to the native font data.
+   *
+   * This field is manipulated by native code. Don't change or remove
+   * without adjusting the native code.
+   */
+  private Pointer nativeFont;
+
   private native void initState ();
   private native void dispose ();
   private native void setFont (String family, int style, int size);
@@ -351,7 +365,7 @@ public class GdkFontPeer extends ClasspathFontPeer
     return NameDecoder.getName(nameTable, name, locale);
   }
 
-  public boolean canDisplay (Font font, char c)
+  public boolean canDisplay (Font font, int c)
   {
     // FIXME: inquire with pango
     return true;
