@@ -44,12 +44,9 @@ import gnu.java.security.Registry;
  * The implementation of the Square <i>Service Provider Interface</i>
  * (<b>SPI</b>) adapter.
  */
-public final class SquareSpi extends CipherAdapter
+public final class SquareSpi
+    extends CipherAdapter
 {
-
-  // Constructors.
-  // --------------------------------------------------------------------
-
   public SquareSpi()
   {
     super(Registry.SQUARE_CIPHER);
