@@ -216,7 +216,7 @@ public class Parser
            * The tag validator closes all unclosed elements that are required
            * to have the end (closing) tag.
            *
-           * @param element The tag being fictionally (forcibly) closed.
+           * @param tElement The tag being fictionally (forcibly) closed.
            */
           protected void handleSupposedEndTag(Element tElement)
           {
@@ -234,7 +234,7 @@ public class Parser
            * assigned to the empty one, the previous value is
            * restored before return.
            *
-           * @param element The tag being fictionally (forcibly) closed.
+           * @param tElement The tag being fictionally (forcibly) closed.
            */
           protected void handleSupposedStartTag(Element tElement)
           {
@@ -740,7 +740,7 @@ public class Parser
    * Handle the tag with no content, like &lt;br&gt;. The method is
    * called for the elements that, in accordance with the current DTD,
    * has an empty content.
-   * @param The tag being handled.
+   * @param tag The tag being handled.
    * @throws javax.swing.text.ChangedCharSetException
    */
   protected void handleEmptyTag(TagElement tag)
@@ -752,7 +752,7 @@ public class Parser
    * The method is called when the HTML closing tag ((like &lt;/table&gt;)
    * is found or if the parser concludes that the one should be present
    * in the current position.
-   * @param The tag
+   * @param tag The tag
    */
   protected void handleEndTag(TagElement tag)
   {
@@ -767,7 +767,7 @@ public class Parser
    * The method is called when the HTML opening tag ((like &lt;table&gt;)
    * is found or if the parser concludes that the one should be present
    * in the current position.
-   * @param The tag
+   * @param tag The tag
    */
   protected void handleStartTag(TagElement tag)
   {
@@ -796,7 +796,7 @@ public class Parser
    * both title starting and closing tags are already behind.
    * The passed argument contains the concatenation of all
    * title text sections.
-   * @param The title text.
+   * @param title The title text.
    */
   protected void handleTitle(char[] title)
   {
@@ -866,7 +866,7 @@ public class Parser
   {
     Object value = HTML.NULL_ATTRIBUTE_VALUE;
 
-    Element e = (Element) dtd.elementHash.get(element.toLowerCase());
+    Element e = dtd.elementHash.get(element.toLowerCase());
     if (e != null)
       {
         AttributeList attr = e.getAttribute(attribute);
@@ -1144,7 +1144,7 @@ public class Parser
    * is found or if the parser concludes that the one should be present
    * in the current position. The method is called immediately before
    * calling the handleStartTag.
-   * @param The tag
+   * @param tag The tag
    */
   protected void startTag(TagElement tag)
                    throws ChangedCharSetException
@@ -1180,7 +1180,7 @@ public class Parser
    * A hooks for operations, preceeding call to handleEmptyTag().
    * Handle the tag with no content, like &lt;br&gt;. As no any
    * nested tags are expected, the tag validator is not involved.
-   * @param The tag being handled.
+   * @param tag The tag being handled.
    */
   private void _handleEmptyTag(TagElement tag)
   {
@@ -1206,7 +1206,7 @@ public class Parser
    * A hooks for operations, preceeding call to handleEndTag().
    * The method is called when the HTML closing tag
    * is found. Calls handleTitle after closing the 'title' tag.
-   * @param The tag
+   * @param tag The tag
    */
   private void _handleEndTag(TagElement tag)
   {
@@ -1252,7 +1252,7 @@ public class Parser
    * The method is called when the HTML opening tag ((like &lt;table&gt;)
    * is found.
    * Package-private to avoid an accessor method.
-   * @param The tag
+   * @param tag The tag
    */
   void _handleStartTag(TagElement tag)
   {
@@ -1326,7 +1326,7 @@ public class Parser
 
   private TagElement makeTagElement(String name, boolean isSupposed)
   {
-    Element e = (Element) dtd.elementHash.get(name.toLowerCase());
+    Element e = dtd.elementHash.get(name.toLowerCase());
     if (e == null)
       {
         error("Unknown tag <" + name + ">");
