@@ -167,7 +167,7 @@ public class DirectColorModel extends PackedColorModel
   private int extractAndNormalizeSample(int pixel, int component)
   {
     int value = extractAndScaleSample(pixel, component);
-    if (hasAlpha() && isAlphaPremultiplied())
+    if (hasAlpha() && isAlphaPremultiplied() && getAlpha(pixel) != 0)
       value = value*255/getAlpha(pixel);
     return value;
   }
@@ -348,9 +348,24 @@ public class DirectColorModel extends PackedColorModel
   {
     return getComponents(getPixelFromArray(pixel), components, offset);
   }
-  
+
+  /**
+   * Creates a <code>WriteableRaster</code> that has a <code>SampleModel</code>
+   * that is compatible with this <code>ColorModel</code>.
+   *
+   * @param w the width of the writeable raster to create
+   * @param h the height of the writeable raster to create
+   *
+   * @throws IllegalArgumentException if <code>w</code> or <code>h</code>
+   *         is less than or equal to zero
+   */
   public final WritableRaster createCompatibleWritableRaster(int w, int h)
   {
+    // Sun also makes this check here.
+    if(w <= 0 || h <= 0)
+      throw new IllegalArgumentException("width (=" + w + ") and height (="
+                                         + h + ") must be > 0");
+
     SampleModel sm = createCompatibleSampleModel(w, h);
     Point origin = new Point(0, 0);
     return Raster.createWritableRaster(sm, origin);	
@@ -378,20 +393,20 @@ public class DirectColorModel extends PackedColorModel
     return Buffers.getData(buffer);
   }
     
-  public final ColorModel coerceData (WritableRaster raster,
-				      boolean isAlphaPremultiplied)
+  public ColorModel coerceData (WritableRaster raster,
+                                boolean isAlphaPremultiplied)
   {
-    if (this.isAlphaPremultiplied == isAlphaPremultiplied)
+    if (this.isAlphaPremultiplied == isAlphaPremultiplied || !hasAlpha())
       return this;
 	
     /* TODO: provide better implementation based on the
        assumptions we can make due to the specific type of the
        color model. */
-    super.coerceData(raster, isAlphaPremultiplied);
-	
-    return new ComponentColorModel(cspace, bits, hasAlpha(),
-				   isAlphaPremultiplied, // argument
-				   transparency, transferType);
+    super.coerceDataWorker(raster, isAlphaPremultiplied);
+    
+    return new DirectColorModel(cspace, pixel_bits, getRedMask(),
+                                getGreenMask(), getBlueMask(), getAlphaMask(),
+                                isAlphaPremultiplied, transferType);
   } 
 
   public boolean isCompatibleRaster(Raster raster)
@@ -418,3 +433,4 @@ public class DirectColorModel extends PackedColorModel
     return super.toString();
   }
 }
+
