@@ -262,10 +262,9 @@ public class URLClassLoader extends SecureClassLoader
     super(parent);
     this.securityContext = null;
     this.factory = factory;
-    addURLs(urls);
-
-    // If this factory is still not in factoryCache, add it.
+    // If this factory is not yet in factoryCache, add it.
     factoryCache.add(factory);
+    addURLs(urls);
   }
 
   // Methods
@@ -453,7 +452,7 @@ public class URLClassLoader extends SecureClassLoader
   {
     // Compute the name of the package as it may appear in the
     // Manifest.
-    StringBuffer xform = new StringBuffer(name);
+    StringBuilder xform = new StringBuilder(name);
     for (int i = xform.length () - 1; i >= 0; --i)
       if (xform.charAt(i) == '.')
 	xform.setCharAt(i, '/');
@@ -624,10 +623,7 @@ public class URLClassLoader extends SecureClassLoader
       }
     catch (IOException ioe)
       {
-	ClassNotFoundException cnfe;
-	cnfe = new ClassNotFoundException(className + " not found in " + this);
-	cnfe.initCause(ioe);
-	throw cnfe;
+	throw new ClassNotFoundException(className + " not found in " + this, ioe);
       }
   }
   
@@ -645,7 +641,7 @@ public class URLClassLoader extends SecureClassLoader
       {
 	if (thisString == null)
 	  {
-	    StringBuffer sb = new StringBuffer();
+	    StringBuilder sb = new StringBuilder();
 	    sb.append(this.getClass().getName());
 	    sb.append("{urls=[" );
 	    URL[] thisURLs = getURLs();
