@@ -1,5 +1,5 @@
 /* LongBuffer.java -- 
-   Copyright (C) 2002, 2003, 2004  Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003, 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -42,7 +42,7 @@ package java.nio;
  * @since 1.4
  */
 public abstract class LongBuffer extends Buffer
-  implements Comparable
+  implements Comparable<LongBuffer>
 {
   int array_offset;
   long[] backing_buffer;
@@ -273,7 +273,7 @@ public abstract class LongBuffer extends Buffer
   {
     if (obj instanceof LongBuffer)
       {
-        return compareTo (obj) == 0;
+        return compareTo ((LongBuffer) obj) == 0;
       }
 
     return false;
@@ -285,10 +285,8 @@ public abstract class LongBuffer extends Buffer
    * @exception ClassCastException If obj is not an object derived from
    * <code>LongBuffer</code>.
    */
-  public int compareTo (Object obj)
+  public int compareTo (LongBuffer other)
   {
-    LongBuffer other = (LongBuffer) obj;
-
     int num = Math.min(remaining(), other.remaining());
     int pos_this = position();
     int pos_other = other.position();
