@@ -38,6 +38,8 @@ exception statement from your version. */
 
 package java.text;
 
+import gnu.classpath.NotImplementedException;
+
 import java.util.ArrayList;
 import java.util.HashMap;
 
@@ -510,7 +512,7 @@ main_parse_loop:
 	    int idx;
 
 	    // Parse the subrules but do not iterate through all
-	    // sublist. This is the priviledge of the first call.
+	    // sublist. This is the privilege of the first call.
 	    idx = subParseString(true, sorted_rules, base_offset+i, subrules);
     
 	    // Merge new parsed rules into the list.
@@ -922,15 +924,7 @@ element_loop:
    */
   public CollationElementIterator getCollationElementIterator(CharacterIterator source)
   {
-    StringBuffer expand = new StringBuffer("");
-    
-    // Right now we assume that we will read from the beginning of the string.
-    for (char c = source.first();
-	 c != CharacterIterator.DONE;
-	 c = source.next())
-      decomposeCharacter(c, expand);
-
-    return getCollationElementIterator(expand.toString());
+    return new CollationElementIterator(this, source);
   }
 
   /**
