@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -41,6 +41,8 @@ import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.io.Serializable;
+import java.text.DateFormat;
+import java.text.SimpleDateFormat;
 
 /**
  * <p>
@@ -106,7 +108,7 @@ import java.io.Serializable;
  * @author Andrew John Hughes (gnu_andrew@member.fsf.org)
  */
 public class Date
-    implements Cloneable, Comparable, Serializable
+    implements Cloneable, Comparable<Date>, Serializable
 {
   /**
    * This is the serialization UID for this class
@@ -240,8 +242,8 @@ public class Date
    *             <code>TimeZone</code> instead.
    * @param year the difference between the required year and 1900.
    * @param month the month as a value between 0 and 11.
-   * @param day the day as a value between 0 and 31.
-   * @param hour the hour as a value between 0 and 23, in 24-hour
+   * @param date the day as a value between 0 and 31.
+   * @param hrs the hour as a value between 0 and 23, in 24-hour
    *        clock notation.
    * @param min the minute as a value between 0 and 59.
    * @param sec the second as a value between 0 and 61 (with 60
@@ -351,24 +353,6 @@ public class Date
     return (time < when.time) ? -1 : (time == when.time) ? 0 : 1;
   }
 
-  /**
-   * Compares this Date to another object.  This behaves like
-   * <code>compareTo(Date)</code>, but it takes a generic object
-   * and throws a <code>ClassCastException</code> if obj is
-   * not a <code>Date</code>.
-   * 
-   * @param obj the other date.
-   * @return 0, if the date represented
-   * by obj is exactly the same as the time represented by this
-   * object, a negative if this Date is before the other Date, and
-   * a positive value otherwise.  
-   * @exception ClassCastException if obj is not of type Date.
-   */
-  public int compareTo(Object obj)
-  {
-    return compareTo((Date) obj);
-  }
-
   /**
    * Computes the hash code of this <code>Date</code> as the
    * XOR of the most significant and the least significant
@@ -718,7 +702,7 @@ public class Date
    * </li>
    * </ul>
    *
-   * @param s The String to parse.
+   * @param string The String to parse.
    * @return The time in milliseconds since the epoch.
    * @throws IllegalArgumentException if the string fails to parse.
    * @deprecated Use DateFormat.parse(String)
@@ -738,7 +722,7 @@ public class Date
     boolean localTimezone = true;
 
     // Trim out any nested stuff in parentheses now to make parsing easier.
-    StringBuffer buf = new StringBuffer();
+    StringBuilder buf = new StringBuilder();
     int parenNesting = 0;
     int len = string.length();
     for (int i = 0;  i < len;  i++)
@@ -770,6 +754,7 @@ public class Date
 	  }
 	else if (firstch >= '0' && firstch <= '9')
 	  {
+	    int lastPunct = -1;
 	    while (tok != null && tok.length() > 0)
 	      {
 		int punctOffset = tok.length();
@@ -807,6 +792,13 @@ public class Date
 		    else
 		      minute = num;
 		  }
+		else if (lastPunct == ':' && hour >= 0 && (minute < 0 || second < 0))
+		  {
+		    if (minute < 0)
+		      minute = num;
+		    else
+		      second = num;
+		  }
 	        else if ((num >= 70
 			  && (punct == ' ' || punct == ','
 			      || punct == '/' || punct < 0))
@@ -844,6 +836,7 @@ public class Date
 		  tok = null;
 		else
 		  tok = tok.substring(punctOffset + 1);
+		lastPunct = punct;
 	      }
 	  }
 	else if (firstch >= 'A' && firstch <= 'Z')
@@ -863,7 +856,7 @@ public class Date
 		  hour += 12;
 	      }
 	    else if (parseDayOfWeek(tok))
-	      ; // Ignore it; throw the token away.
+	      { /* Ignore it; throw the token away. */ }
 	    else if (tok.equals("UT") || tok.equals("UTC") || tok.equals("GMT"))
 	      localTimezone = false;
 	    else if (tok.startsWith("UT") || tok.startsWith("GMT"))
