@@ -45,6 +45,7 @@ import java.awt.Insets;
 import java.beans.PropertyChangeListener;
 import java.beans.PropertyChangeSupport;
 import java.io.Serializable;
+import java.util.Enumeration;
 import java.util.Locale;
 
 import javax.swing.border.Border;
@@ -117,6 +118,95 @@ public class UIManager implements Serializable
     }
   }
 
+  /**
+   * A UIDefaults subclass that multiplexes between itself and a 'fallback'
+   * UIDefaults instance. This is used to protect the L&F UIDefaults from beeing
+   * overwritten by applications.
+   */
+  private static class MultiplexUIDefaults
+    extends UIDefaults
+  {
+    private class MultiplexEnumeration
+      implements Enumeration
+    {
+      Enumeration[] enums;
+      int i;
+      MultiplexEnumeration(Enumeration e1, Enumeration e2)
+      {
+        enums = new Enumeration[]{ e1, e2 };
+        i = 0;
+      }
+
+      public boolean hasMoreElements()
+      {
+        return enums[i].hasMoreElements() || i < enums.length - 1;
+      }
+
+      public Object nextElement()
+      {
+        Object val = enums[i].nextElement();
+        if (! enums[i].hasMoreElements() && i < enums.length - 1)
+          i++;
+        return val;
+      }
+        
+    }
+
+    UIDefaults fallback;
+
+    /**
+     * Creates a new <code>MultiplexUIDefaults</code> instance with 
+     * <code>d</code> as the fallback defaults.
+     * 
+     * @param d  the fallback defaults (<code>null</code> not permitted).
+     */
+    MultiplexUIDefaults(UIDefaults d)
+    {
+      if (d == null) 
+        throw new NullPointerException();
+      fallback = d;
+    }
+
+    public Object get(Object key)
+    {
+      Object val = super.get(key);
+      if (val == null)
+        val = fallback.get(key);
+      return val;
+    }
+
+    public Object get(Object key, Locale l)
+    {
+      Object val = super.get(key, l);
+      if (val == null)
+        val = fallback.get(key, l);
+      return val;
+    }
+
+    public Object remove(Object key)
+    {
+      Object val = super.remove(key);
+      if (val == null)
+        val = fallback.remove(key);
+      return val;
+    }
+
+    public int size()
+    {
+      return super.size() + fallback.size();
+    }
+
+    public Enumeration keys()
+    {
+      return new MultiplexEnumeration(super.keys(), fallback.keys());
+    }
+
+    public Enumeration elements()
+    {
+      return new MultiplexEnumeration(super.elements(), fallback.elements());
+    }
+  }
+
   private static final long serialVersionUID = -5547433830339189365L;
 
   /** The installed look and feel(s). */
@@ -131,12 +221,9 @@ public class UIManager implements Serializable
   /** The current look and feel. */
   static LookAndFeel currentLookAndFeel;
   
-  static UIDefaults currentUIDefaults;
+  static MultiplexUIDefaults currentUIDefaults;
 
-  /**
-   * UIDefaults set by the user.
-   */
-  static UIDefaults userUIDefaults;
+  static UIDefaults lookAndFeelDefaults;
 
   /** Property change listener mechanism. */
   static PropertyChangeSupport listeners
@@ -145,22 +232,22 @@ public class UIManager implements Serializable
   static
   {
     String defaultlaf = System.getProperty("swing.defaultlaf");
-    try {
-      if (defaultlaf != null)
-        {
-          Class lafClass = Class.forName(defaultlaf);
-          LookAndFeel laf = (LookAndFeel) lafClass.newInstance();
-          setLookAndFeel(laf);
-        }
-      else
-        {
-          setLookAndFeel(new MetalLookAndFeel());
-        }
-    }
+    try 
+      {
+        if (defaultlaf != null)
+          {
+            setLookAndFeel(defaultlaf);
+          }
+        else
+          {
+            setLookAndFeel(new MetalLookAndFeel());
+          }
+      }
     catch (Exception ex)
       {
         System.err.println("cannot initialize Look and Feel: " + defaultlaf);
         System.err.println("error: " + ex.toString());
+        ex.printStackTrace();
         System.err.println("falling back to Metal Look and Feel");
         try
           {
@@ -311,12 +398,7 @@ public class UIManager implements Serializable
    */
   public static Object get(Object key)
   {
-    Object val = null;
-    if (userUIDefaults != null)
-      val = userUIDefaults.get(key);
-    if (val == null)
-      val = getLookAndFeelDefaults().get(key);
-    return val;
+    return getDefaults().get(key);
   }
 
   /**
@@ -326,73 +408,129 @@ public class UIManager implements Serializable
    * @param key  the key.
    * 
    * @return The object.
+   * 
+   * @since 1.4
    */
   public static Object get(Object key, Locale locale)
   {
-    Object val = null;
-    if (userUIDefaults != null)
-      val = userUIDefaults.get(key, locale);
-    if (val == null)
-      val = getLookAndFeelDefaults().get(key, locale);
-    return val;
+    return getDefaults().get(key, locale);
   }
 
   /**
-   * Returns a boolean value from the defaults table,
-   * <code>false</code> if key is not present.
+   * Returns a boolean value from the defaults table.  If there is no value
+   * for the specified key, or the value is not an instance of {@link Boolean},
+   * this method returns <code>false</code>.
+   * 
+   * @param key  the key (<code>null</code> not permitted).
    *
+   * @return The boolean value associated with the specified key.
+   * 
+   * @throws NullPointerException if <code>key</code> is <code>null</code>.
+   * 
    * @since 1.4
    */
   public static boolean getBoolean(Object key)
   {
-    Boolean value = (Boolean) get(key);
-    return value != null ? value.booleanValue() : false;
+    Object value = get(key);
+    if (value instanceof Boolean) 
+      return ((Boolean) value).booleanValue();
+    return false;
   }
   
   /**
-   * Returns a boolean value from the defaults table,
-   * <code>false</code> if key is not present.
+   * Returns a boolean value from the defaults table.  If there is no value
+   * for the specified key, or the value is not an instance of {@link Boolean},
+   * this method returns <code>false</code>.
+   * 
+   * @param key  the key (<code>null</code> not permitted).
+   * @param locale  the locale.
    *
+   * @return The boolean value associated with the specified key.
+   * 
+   * @throws NullPointerException if <code>key</code> is <code>null</code>.
+   * 
    * @since 1.4
    */
   public static boolean getBoolean(Object key, Locale locale)
   {
-    Boolean value = (Boolean) get(key, locale);
-    return value != null ? value.booleanValue() : false;
+    Object value = get(key, locale);
+    if (value instanceof Boolean) 
+      return ((Boolean) value).booleanValue();
+    return false;
   }
     
   /**
    * Returns a border from the defaults table. 
+   * 
+   * @param key  the key (<code>null</code> not permitted).
+   * 
+   * @return The border associated with the given key, or <code>null</code>.
+   * 
+   * @throws NullPointerException if <code>key</code> is <code>null</code>.
    */
   public static Border getBorder(Object key)
   {
-    return (Border) get(key);
+    Object value = get(key);
+    if (value instanceof Border) 
+      return (Border) value;
+    return null;
   }
     
   /**
-   * Returns a border from the defaults table.
+   * Returns a border from the defaults table. 
+   * 
+   * @param key  the key (<code>null</code> not permitted).
+   * @param locale  the locale.
+   * 
+   * @return The border associated with the given key, or <code>null</code>.
+   * 
+   * @throws NullPointerException if <code>key</code> is <code>null</code>.
    *
    * @since 1.4
    */
   public static Border getBorder(Object key, Locale locale)
   {
-    return (Border) get(key, locale);
+    Object value = get(key, locale);
+    if (value instanceof Border) 
+      return (Border) value;
+    return null;
   }
     
   /**
    * Returns a drawing color from the defaults table. 
+   * 
+   * @param key  the key (<code>null</code> not permitted).
+   * 
+   * @return The color associated with the given key, or <code>null</code>.
+   * 
+   * @throws NullPointerException if <code>key</code> is <code>null</code>.
    */
   public static Color getColor(Object key)
   {
-    return (Color) get(key);
+    Object value = get(key);
+    if (value instanceof Color) 
+      return (Color) value;
+    return null;
   }
 
   /**
    * Returns a drawing color from the defaults table. 
+   * 
+   * @param key  the key (<code>null</code> not permitted).
+   * @param locale  the locale.
+   * 
+   * @return The color associated with the given key, or <code>null</code>.
+   * 
+   * @throws NullPointerException if <code>key</code> is <code>null</code>.
+   * 
+   * @since 1.4
    */
   public static Color getColor(Object key, Locale locale)
   {
-    return (Color) get(key);
+    Object value = get(key, locale);
+    if (value instanceof Color) 
+      return (Color) value;
+    return null;
   }
 
   /**
@@ -413,23 +551,45 @@ public class UIManager implements Serializable
    */
   public static UIDefaults getDefaults()
   {
+    if (currentUIDefaults == null)
+      currentUIDefaults = new MultiplexUIDefaults(new UIDefaults());
     return currentUIDefaults;
   }
 
   /**
    * Returns a dimension from the defaults table. 
+   * 
+   * @param key  the key (<code>null</code> not permitted).
+   * 
+   * @return The color associated with the given key, or <code>null</code>.
+   * 
+   * @throws NullPointerException if <code>key</code> is <code>null</code>.
    */
   public static Dimension getDimension(Object key)
   {
-    return (Dimension) get(key);
+    Object value = get(key);
+    if (value instanceof Dimension) 
+      return (Dimension) value;
+    return null;
   }
 
   /**
    * Returns a dimension from the defaults table. 
+   * 
+   * @param key  the key (<code>null</code> not permitted).
+   * @param locale  the locale.
+   * 
+   * @return The color associated with the given key, or <code>null</code>.
+   * 
+   * @throws NullPointerException if <code>key</code> is <code>null</code>.
+   * @since 1.4
    */
   public static Dimension getDimension(Object key, Locale locale)
   {
-    return (Dimension) get(key, locale);
+    Object value = get(key, locale);
+    if (value instanceof Dimension) 
+      return (Dimension) value;
+    return null;
   }
 
   /**
@@ -439,10 +599,17 @@ public class UIManager implements Serializable
    * @param key an Object that specifies the font. Typically,
    *        this is a String such as
    *        <code>TitledBorder.font</code>.
+   *        
+   * @return The font associated with the given key, or <code>null</code>.
+   * 
+   * @throws NullPointerException if <code>key</code> is <code>null</code>.
    */
   public static Font getFont(Object key)
   {
-    return (Font) get(key);
+    Object value = get(key);
+    if (value instanceof Font) 
+      return (Font) value;
+    return null;
   }
 
   /**
@@ -452,30 +619,66 @@ public class UIManager implements Serializable
    * @param key an Object that specifies the font. Typically,
    *        this is a String such as
    *        <code>TitledBorder.font</code>.
+   * @param locale  the locale.
+   *        
+   * @return The font associated with the given key, or <code>null</code>.
+   * 
+   * @throws NullPointerException if <code>key</code> is <code>null</code>.
+   *        
+   * @since 1.4
    */
   public static Font getFont(Object key, Locale locale)
   {
-    return (Font) get(key ,locale);
+    Object value = get(key, locale);
+    if (value instanceof Font) 
+      return (Font) value;
+    return null;
   }
 
   /**
-   * Returns an Icon from the defaults table.
+   * Returns an icon from the defaults table. 
+   * 
+   * @param key  the key (<code>null</code> not permitted).
+   * 
+   * @return The icon associated with the given key, or <code>null</code>.
+   * 
+   * @throws NullPointerException if <code>key</code> is <code>null</code>.
    */
   public static Icon getIcon(Object key)
   {
-    return (Icon) get(key);
+    Object value = get(key);
+    if (value instanceof Icon) 
+      return (Icon) value;
+    return null;
   }
   
   /**
-   * Returns an Icon from the defaults table.
+   * Returns an icon from the defaults table. 
+   * 
+   * @param key  the key (<code>null</code> not permitted).
+   * @param locale  the locale.
+   * 
+   * @return The icon associated with the given key, or <code>null</code>.
+   * 
+   * @throws NullPointerException if <code>key</code> is <code>null</code>.
+   * @since 1.4
    */
   public static Icon getIcon(Object key, Locale locale)
   {
-    return (Icon) get(key, locale);
+    Object value = get(key, locale);
+    if (value instanceof Icon) 
+      return (Icon) value;
+    return null;
   }
   
   /**
    * Returns an Insets object from the defaults table.
+   * 
+   * @param key  the key (<code>null</code> not permitted).
+   * 
+   * @return The insets associated with the given key, or <code>null</code>.
+   * 
+   * @throws NullPointerException if <code>key</code> is <code>null</code>.   
    */
   public static Insets getInsets(Object key)
   {
@@ -488,6 +691,14 @@ public class UIManager implements Serializable
 
   /**
    * Returns an Insets object from the defaults table.
+   * 
+   * @param key  the key (<code>null</code> not permitted).
+   * @param locale  the locale.
+   * 
+   * @return The insets associated with the given key, or <code>null</code>.
+   * 
+   * @throws NullPointerException if <code>key</code> is <code>null</code>.   
+   * @since 1.4
    */
   public static Insets getInsets(Object key, Locale locale)
   {
@@ -509,20 +720,41 @@ public class UIManager implements Serializable
     return installed;
   }
 
+  /**
+   * Returns the integer value of the {@link Integer} associated with the
+   * given key.  If there is no value, or the value is not an instance of
+   * {@link Integer}, this method returns 0.
+   * 
+   * @param key  the key (<code>null</code> not permitted).
+   * 
+   * @return The integer value associated with the given key, or 0.
+   */
   public static int getInt(Object key)
   {
-    Integer x = (Integer) get(key);
-    if (x == null)
-      return 0;
-    return x.intValue();
+    Object x = get(key);
+    if (x instanceof Integer)
+      return ((Integer) x).intValue();
+    return 0;
   }
 
+  /**
+   * Returns the integer value of the {@link Integer} associated with the
+   * given key.  If there is no value, or the value is not an instance of
+   * {@link Integer}, this method returns 0.
+   * 
+   * @param key  the key (<code>null</code> not permitted).
+   * @param locale  the locale.
+   * 
+   * @return The integer value associated with the given key, or 0.
+   * 
+   * @since 1.4
+   */
   public static int getInt(Object key, Locale locale)
   {
-    Integer x = (Integer) get(key, locale);
-    if (x == null)
-      return 0;
-    return x.intValue();
+    Object x = get(key, locale);
+    if (x instanceof Integer)
+      return ((Integer) x).intValue();
+    return 0;
   }
 
   /**
@@ -545,23 +777,42 @@ public class UIManager implements Serializable
    */
   public static UIDefaults getLookAndFeelDefaults()
   {
-    return currentUIDefaults;
+    return lookAndFeelDefaults;
   }
 
   /**
-   * Returns a string from the defaults table.
+   * Returns the {@link String} associated with the given key.  If the value 
+   * is not a {@link String}, this method returns <code>null</code>.
+   * 
+   * @param key  the key (<code>null</code> not permitted).
+   * 
+   * @return The string associated with the given key, or <code>null</code>.
    */
   public static String getString(Object key)
   {
-    return (String) get(key);
+    Object s = get(key);
+    if (s instanceof String)
+      return (String) s;
+    return null;
   }
   
   /**
-   * Returns a string from the defaults table.
+   * Returns the {@link String} associated with the given key.  If the value 
+   * is not a {@link String}, this method returns <code>null</code>.
+   * 
+   * @param key  the key (<code>null</code> not permitted).
+   * @param locale  the locale.
+   * 
+   * @return The string associated with the given key, or <code>null</code>.
+   * 
+   * @since 1.4
    */
   public static String getString(Object key, Locale locale)
   {
-    return (String) get(key, locale);
+    Object s = get(key, locale);
+    if (s instanceof String)
+      return (String) s;
+    return null;
   }
   
   /**
@@ -586,13 +837,7 @@ public class UIManager implements Serializable
    */
   public static ComponentUI getUI(JComponent target)
   {
-    ComponentUI ui = null;
-    if (userUIDefaults != null
-        && userUIDefaults.get(target.getUIClassID()) != null)
-      ui = userUIDefaults.getUI(target);
-    if (ui == null)
-      ui = currentUIDefaults.getUI(target);
-    return ui;
+    return getDefaults().getUI(target);
   }
 
   /**
@@ -621,14 +866,13 @@ public class UIManager implements Serializable
 
   /**
    * Stores an object in the defaults table.
+   * 
+   * @param key  the key.
+   * @param value  the value.
    */
   public static Object put(Object key, Object value)
   {
-    Object old = get(key);
-    if (userUIDefaults == null)
-      userUIDefaults = new UIDefaults();
-    userUIDefaults.put(key, value);
-    return old;
+    return getDefaults().put(key, value);
   }
 
   /**
@@ -653,7 +897,8 @@ public class UIManager implements Serializable
     throws UnsupportedLookAndFeelException
   {
     if (newLookAndFeel != null && ! newLookAndFeel.isSupportedLookAndFeel())
-      throw new UnsupportedLookAndFeelException(newLookAndFeel.getName());
+      throw new UnsupportedLookAndFeelException(newLookAndFeel.getName()
+                                         + " not supported on this platform");
     LookAndFeel oldLookAndFeel = currentLookAndFeel;
     if (oldLookAndFeel != null)
       oldLookAndFeel.uninitialize();
@@ -663,7 +908,12 @@ public class UIManager implements Serializable
     if (newLookAndFeel != null)
       {
         newLookAndFeel.initialize();
-        currentUIDefaults = newLookAndFeel.getDefaults();
+        lookAndFeelDefaults = newLookAndFeel.getDefaults();
+        if (currentUIDefaults == null)
+          currentUIDefaults =
+            new MultiplexUIDefaults(lookAndFeelDefaults);
+        else
+          currentUIDefaults.fallback = lookAndFeelDefaults;
       }
     else
       {
@@ -688,7 +938,8 @@ public class UIManager implements Serializable
     throws ClassNotFoundException, InstantiationException, IllegalAccessException,
     UnsupportedLookAndFeelException
   {
-    Class c = Class.forName(className);
+    Class c = Class.forName(className, true,
+                            Thread.currentThread().getContextClassLoader());
     LookAndFeel a = (LookAndFeel) c.newInstance(); // throws class-cast-exception
     setLookAndFeel(a);
   }
