@@ -67,7 +67,7 @@ public class DefaultMutableTreeNode
    * An empty enumeration, returned by {@link #children()} if a node has no
    * children.
    */
-  public static final Enumeration EMPTY_ENUMERATION =
+  public static final Enumeration<TreeNode> EMPTY_ENUMERATION =
     EmptyEnumeration.getInstance();
 
   /**
@@ -78,7 +78,7 @@ public class DefaultMutableTreeNode
   /**
    * The child nodes for this node (may be empty).
    */
-  protected Vector children = new Vector();
+  protected Vector<MutableTreeNode> children = new Vector<MutableTreeNode>();
 
   /**
    * userObject
@@ -202,7 +202,7 @@ public class DefaultMutableTreeNode
    */
   public void remove(int index)
   {
-    MutableTreeNode child = (MutableTreeNode) children.remove(index);
+    MutableTreeNode child = children.remove(index);
     child.setParent(null);
   }
 
@@ -480,7 +480,7 @@ public class DefaultMutableTreeNode
   public TreeNode getSharedAncestor(DefaultMutableTreeNode node)
   {
     TreeNode current = this;
-    ArrayList list = new ArrayList();
+    ArrayList<TreeNode> list = new ArrayList<TreeNode>();
 
     while (current != null)
       {
@@ -527,7 +527,7 @@ public class DefaultMutableTreeNode
         || children.size() == 0)
       return 0;
 
-    Stack stack = new Stack();
+    Stack<Integer> stack = new Stack<Integer>();
     stack.push(new Integer(0));
     TreeNode node = getChildAt(0);
     int depth = 0;
@@ -553,7 +553,7 @@ public class DefaultMutableTreeNode
               {
                 node = node.getParent();
                 size = node.getChildCount();
-                index = ((Integer) stack.pop()).intValue() + 1;
+                index = stack.pop().intValue() + 1;
                 current--;
               }
             while (index >= size
@@ -765,7 +765,7 @@ public class DefaultMutableTreeNode
       throw new IllegalArgumentException();
     
     TreeNode parent = this;
-    Vector nodes = new Vector();
+    Vector<TreeNode> nodes = new Vector<TreeNode>();
     nodes.add(this);
 
     while (parent != node && parent != null)
@@ -887,8 +887,7 @@ public class DefaultMutableTreeNode
       return false;
     if (node == this)
       return true;
-    return (node.getParent() == getParent()
-            && getParent() != null);
+    return node.getParent() == getParent() && getParent() != null;
   }
 
   /**
@@ -1053,10 +1052,10 @@ public class DefaultMutableTreeNode
   /** Provides an enumeration of a tree in breadth-first traversal
    * order.
    */
-  static class BreadthFirstEnumeration implements Enumeration
+  static class BreadthFirstEnumeration implements Enumeration<TreeNode>
   {
 
-      LinkedList queue = new LinkedList();
+      LinkedList<TreeNode> queue = new LinkedList<TreeNode>();
 
       BreadthFirstEnumeration(TreeNode node)
       {
@@ -1068,14 +1067,16 @@ public class DefaultMutableTreeNode
           return !queue.isEmpty();
       }
 
-      public Object nextElement()
+      @SuppressWarnings("unchecked")
+      public TreeNode nextElement()
       {
-          if(queue.isEmpty())
+          if (queue.isEmpty())
               throw new NoSuchElementException("No more elements left.");
 
-          TreeNode node = (TreeNode) queue.removeFirst();
+          TreeNode node = queue.removeFirst();
 
-          Enumeration children = node.children();
+          Enumeration<TreeNode> children =
+	    (Enumeration<TreeNode>) node.children();
           while (children.hasMoreElements())
               queue.add(children.nextElement());
 
@@ -1086,16 +1087,18 @@ public class DefaultMutableTreeNode
   /** Provides an enumeration of a tree traversing it
    * preordered.
    */
-  static class PreorderEnumeration implements Enumeration
+  static class PreorderEnumeration implements Enumeration<TreeNode>
   {
 	  TreeNode next;
 
-      Stack childrenEnums = new Stack();
+      Stack<Enumeration<TreeNode>> childrenEnums =
+	new Stack<Enumeration<TreeNode>>();
 
+      @SuppressWarnings("unchecked")
       PreorderEnumeration(TreeNode node)
       {
           next = node;
-          childrenEnums.push(node.children());
+          childrenEnums.push((Enumeration<TreeNode>) node.children());
       }
 
       public boolean hasMoreElements()
@@ -1103,14 +1106,14 @@ public class DefaultMutableTreeNode
           return next != null;
       }
 
-      public Object nextElement()
+      public TreeNode nextElement()
       {
-          if( next == null )
+          if (next == null)
               throw new NoSuchElementException("No more elements left.");
 
-          Object current = next;
+          TreeNode current = next;
 
-          Enumeration children = (Enumeration) childrenEnums.peek();
+          Enumeration<TreeNode> children = childrenEnums.peek();
 
           // Retrieves the next element.
           next = traverse(children);
@@ -1118,13 +1121,14 @@ public class DefaultMutableTreeNode
           return current;
       }
 
-      private TreeNode traverse(Enumeration children)
+      @SuppressWarnings("unchecked")
+      private TreeNode traverse(Enumeration<TreeNode> children)
       {
           // If more children are available step down.
-          if( children.hasMoreElements() )
+          if (children.hasMoreElements())
           {
-              TreeNode child = (TreeNode) children.nextElement();
-              childrenEnums.push(child.children());
+              TreeNode child = children.nextElement();
+              childrenEnums.push((Enumeration<TreeNode>) child.children());
 
               return child;
           }
@@ -1134,11 +1138,11 @@ public class DefaultMutableTreeNode
 
           // If there are no more levels left, there is no next
           // element to return.
-          if ( childrenEnums.isEmpty() )
+          if (childrenEnums.isEmpty())
               return null;
           else
           {
-              return traverse((Enumeration) childrenEnums.peek());
+              return traverse(childrenEnums.peek());
           }
       }
    }
@@ -1146,16 +1150,18 @@ public class DefaultMutableTreeNode
   /** Provides an enumeration of a tree traversing it
    * postordered (= depth-first).
    */
-   static class PostorderEnumeration implements Enumeration
+   static class PostorderEnumeration implements Enumeration<TreeNode>
    {
 
-       Stack nodes = new Stack();
-       Stack childrenEnums = new Stack();
+       Stack<TreeNode> nodes = new Stack<TreeNode>();
+       Stack<Enumeration<TreeNode>> childrenEnums =
+	 new Stack<Enumeration<TreeNode>>();
 
+       @SuppressWarnings("unchecked")
        PostorderEnumeration(TreeNode node)
        {
            nodes.push(node);
-           childrenEnums.push(node.children());
+           childrenEnums.push((Enumeration<TreeNode>) node.children());
        }
 
        public boolean hasMoreElements()
@@ -1163,24 +1169,26 @@ public class DefaultMutableTreeNode
            return !nodes.isEmpty();
        }
 
-       public Object nextElement()
+       public TreeNode nextElement()
        {
-           if( nodes.isEmpty() )
+           if (nodes.isEmpty())
                throw new NoSuchElementException("No more elements left!");
 
-           Enumeration children = (Enumeration) childrenEnums.peek();
+           Enumeration<TreeNode> children = childrenEnums.peek();
 
            return traverse(children);
        }
 
-       private Object traverse(Enumeration children)
+       @SuppressWarnings("unchecked")
+       private TreeNode traverse(Enumeration<TreeNode> children)
        {
-           if ( children.hasMoreElements() )
+           if (children.hasMoreElements())
            {
-               TreeNode node = (TreeNode) children.nextElement();
+               TreeNode node = children.nextElement();
                nodes.push(node);
 
-               Enumeration newChildren = node.children();
+               Enumeration<TreeNode> newChildren =
+		 (Enumeration<TreeNode>) node.children();
                childrenEnums.push(newChildren);
 
                return traverse(newChildren);
@@ -1191,7 +1199,7 @@ public class DefaultMutableTreeNode
 
                // Returns the node whose children
                // have all been visited. (= postorder)
-               Object next = nodes.peek();
+               TreeNode next = nodes.peek();
                nodes.pop();
 
                return next;
